import { jsx, jsxs } from '@emotion/react/jsx-runtime';
import { Box, Text, InputGroup, Input, InputRightElement, Spinner, IconButton, Icon } from '@chakra-ui/react';
import { nanoid } from 'nanoid';
import * as React from 'react';
import React__default from 'react';
import * as iconsList from '@feedloop/icon';
import { TargetLock } from '@feedloop/icon';
import { createContext as createContext$1, useContext, useContextSelector } from 'use-context-selector';
import { enablePatches, produce, applyPatches } from 'immer';
import { v4 } from 'uuid';
import 'filtrex';
import memoize from 'fast-memoize';
import dayjs from 'dayjs';
import { buildExpressionParser } from '@feedloop/expression-parser';
import { mergeConfig, javascriptConfig, evaluate } from '@feedloop/formula-editor';
import { __awaiter } from '/home/arrizalamin/Work/feedloop/qore-apps/node_modules/tslib/tslib.es6.js';
import 'react-router-dom';
import { QoreClient } from '@qorebase/client';
import createQoreHooks from '@qorebase/react';
import { motion } from 'framer-motion';
import '@chakra-ui/icons';
import 'react-tracking';
import { keyframes } from '@emotion/react';
import '/home/arrizalamin/Work/feedloop/qore-apps/node_modules/dot-object/index.js';
import '@qorebase/sdk';
import '/home/arrizalamin/Work/feedloop/qore-apps/node_modules/stable-hash/dist/index.js';
import 'swr';

enablePatches();
function createContext(initialState, createActions, options = {}) {
    const Context = createContext$1({
        state: initialState,
        // eslint-disable-next-line @typescript-eslint/no-unused-vars -- for type inference
        setState: (modifier) => { },
        hasUndo: false,
        hasRedo: false,
        undo: () => { },
        redo: () => { },
        clearStack: () => { },
        // eslint-disable-next-line @typescript-eslint/no-unused-vars -- for type inference
        batchActions: (callback) => { },
        isDirty: false,
        clean: () => { },
    });
    const useContext$1 = () => useContext(Context);
    function useSelectState(selector) {
        return useContextSelector(Context, ({ state }) => selector(state));
    }
    const useActions = () => {
        const setState = useContextSelector(Context, ({ setState }) => setState);
        return React__default.useMemo(() => createActions(setState), [setState]);
    };
    const useHistory = () => useContextSelector(Context, ({ hasUndo, hasRedo, undo, redo, clearStack }) => ({
        hasUndo,
        hasRedo,
        undo,
        redo,
        clear: clearStack,
    }));
    const useBatchActions = () => useContextSelector(Context, ({ batchActions }) => batchActions);
    const useDirty = () => useContextSelector(Context, ({ isDirty, clean }) => ({ isDirty, clean }));
    const Provider = (props) => {
        var _a;
        const [state, setState] = React__default.useState((_a = props.value) !== null && _a !== void 0 ? _a : initialState);
        const [stack, setStack] = React__default.useState({
            undos: [],
            redos: [],
            batchIndex: null,
        });
        const [isDirty, setDirty] = React__default.useState(false);
        const setImmerState = React__default.useCallback((modifier, track = true) => {
            setState((state) => produce(state, modifier, (patches, inversePatches) => {
                if (patches.length === 0)
                    return;
                if (options.trackPaths &&
                    !options.trackPaths.some((trackPath) => patches.every((patch) => patch.path.join(".").startsWith(trackPath))))
                    return;
                if (track && options.trackChanges) {
                    setDirty(true);
                }
                if (options.enableHistory) {
                    setStack((stack) => {
                        return Object.assign(Object.assign({}, stack), { undos: stack.undos.concat({
                                patches,
                                inversePatches,
                            }), redos: [] });
                    });
                }
            }));
        }, []);
        const hasUndo = stack.undos.length > 0;
        const hasRedo = stack.redos.length > 0;
        const undo = React__default.useCallback(() => setStack((stack) => {
            const patchGroup = stack.undos.pop();
            if (!patchGroup)
                return stack;
            setState((state) => applyPatches(state, patchGroup.inversePatches));
            return Object.assign(Object.assign({}, stack), { redos: stack.redos.concat(patchGroup) });
        }), []);
        const redo = React__default.useCallback(() => setStack((stack) => {
            const patchGroup = stack.redos.pop();
            if (!patchGroup)
                return stack;
            setState((state) => applyPatches(state, patchGroup.patches));
            return Object.assign(Object.assign({}, stack), { undos: stack.undos.concat(patchGroup) });
        }), []);
        const batchActions = React__default.useCallback((callback) => {
            setStack((stack) => (Object.assign(Object.assign({}, stack), { batchIndex: stack.undos.length })));
            callback();
        }, []);
        React__default.useEffect(() => {
            const { undos, batchIndex } = stack;
            if (batchIndex === null)
                return;
            const batchPatchGroup = undos.slice(batchIndex).reduce((patchGroup, { patches, inversePatches }) => ({
                patches: patchGroup.patches.concat(patches),
                inversePatches: inversePatches.concat(patchGroup.inversePatches),
            }), {
                patches: [],
                inversePatches: [],
            });
            setStack({
                undos: undos.slice(0, batchIndex).concat(batchPatchGroup),
                redos: [],
                batchIndex: null,
            });
        }, [stack.batchIndex]);
        const clearStack = React__default.useCallback(() => setStack({ undos: [], redos: [], batchIndex: null }), []);
        const clean = React__default.useCallback(() => setDirty(false), []);
        const value = React__default.useMemo(() => ({
            state,
            setState: setImmerState,
            hasUndo,
            hasRedo,
            undo,
            redo,
            clearStack,
            batchActions,
            isDirty,
            clean,
        }), [state, setImmerState, stack, hasUndo, hasRedo]);
        return jsx(Context.Provider, Object.assign({ value: value }, { children: props.children }));
    };
    return {
        useContext: useContext$1,
        useSelectState,
        useActions,
        useHistory,
        useBatchActions,
        useDirty,
        Provider,
    };
}

const regex$1 = /@\[[^*]+?]\(([^*]+?)\)/gim;
const filtrexToFormula = (filtrexString) => {
    const formulaInnerString = filtrexString.replaceAll(regex$1, (match, id) => `{{${id}}}`);
    return formulaInnerString.startsWith("=")
        ? `Filtrex(${JSON.stringify(formulaInnerString.slice(1))})`
        : `"${formulaInnerString}"`;
};
const showConditionToConditionGroup = (condition) => Array.isArray(condition)
    ? {
        type: "all",
        conditions: condition.map((condition) => ({
            id: nanoid(),
            formula: `(${filtrexToFormula(condition.value)}) == (${filtrexToFormula(condition.equals)})`,
        })),
    }
    : condition;

/* eslint-disable fp/no-delete -- immer */
const Home = buildPage();
Home.type = "navigation";
const initialSchema = {
    tables: [],
    functions: [],
    views: [],
    insights: [],
    queries: [],
};
const initialApp = {
    name: "Application",
    description: "",
    icon: "",
    theme_color: "#049fdc",
    config: {
        projectID: "unknown",
        appID: "",
        endpoint: "",
        id: v4(),
        initialPage: {
            entryPage: "",
            loggedInPage: "",
        },
    },
    integrations: {},
    maps: {
        component: {
            login_component: {
                id: "login_component",
                componentID: "Login",
                options: {},
                properties: {},
                showCondition: { type: "all", conditions: [] },
                source: {
                    type: "list",
                    params: {},
                    sorts: {},
                },
                metadata: {},
            },
            register_text_input: {
                componentID: "Text Input",
                options: {},
                showCondition: { type: "all", conditions: [] },
                id: "register_text_input",
                properties: {
                    label: "Email",
                    placeholder: "Enter your email here",
                    hidden: false,
                    defaultValue: "",
                    data: {
                        inputID: "external_id",
                        field: {
                            type: "string",
                            options: {
                                format: "text",
                            },
                        },
                        validation: {
                            required: false,
                        },
                        type: "field",
                    },
                },
                source: {
                    type: "record",
                    params: {},
                    rowId: null,
                    defaultRow: { type: "first-row" },
                },
                metadata: {},
            },
            register_password_input: {
                componentID: "Password",
                options: {},
                showCondition: { type: "all", conditions: [] },
                id: "register_password_input",
                properties: {
                    title: "Password",
                    placeholder: "Enter your password here",
                    confirmPassword: true,
                    confirmTitle: "Confirm Password",
                    confirmPlaceholder: "Reenter your password here",
                    data: {
                        inputID: "password",
                        field: {
                            type: "string",
                            options: {
                                format: "text",
                            },
                        },
                        validation: {
                            required: true,
                        },
                        type: "field",
                    },
                },
                source: {
                    type: "record",
                    params: {},
                    rowId: null,
                    defaultRow: { type: "first-row" },
                },
                metadata: {},
            },
            register_role_input: {
                componentID: "Role",
                options: {},
                showCondition: { type: "all", conditions: [] },
                id: "register_role_input",
                properties: {
                    roleId: "",
                },
                source: {
                    type: "record",
                    params: {},
                    rowId: null,
                    defaultRow: { type: "first-row" },
                },
                metadata: {},
            },
            forbidden_component: {
                id: "forbidden_component",
                componentID: "Forbidden",
                options: {},
                showCondition: { type: "all", conditions: [] },
                properties: {},
                metadata: {},
                source: {
                    type: "record",
                    params: {},
                    rowId: null,
                    defaultRow: { type: "first-row" },
                },
            },
        },
        page: {
            login: Object.assign(Object.assign({}, buildPage()), { id: "login", components: [{ component: "login_component" }], type: "system", title: "Login" }),
            register: Object.assign(Object.assign({}, buildPage()), { id: "register", components: [
                    {
                        component: "register_text_input",
                    },
                    {
                        component: "register_password_input",
                    },
                    {
                        component: "register_role_input",
                    },
                ], type: "system", title: "Register", sourceID: "users", mode: "form" }),
            forbidden: Object.assign(Object.assign({}, buildPage()), { id: "forbidden", components: [{ component: "forbidden_component" }], type: "system", title: "Forbidden" }),
        },
    },
    theme: {
        id: "Light",
        options: { primaryColor: "#1890ff" },
        navigationLayout: "sidebar",
        fonts: {
            heading: ["Inter", ""],
            body: ["Inter", ""],
        },
    },
    schema: initialSchema,
    customSources: {},
    components: [],
};
const initialRuntime = {
    app: initialApp,
    control: { page: "" },
};
createContext(initialRuntime, (modify) => {
    try {
        return {
            setTheme: (theme) => {
                modify((draft) => {
                    draft.app.theme = theme;
                });
            },
            setApp: (app) => {
                modify((draft) => {
                    var _a, _b, _c;
                    const pages = {};
                    for (const page of Object.values(app.maps.page)) {
                        if (page.id) {
                            pages[page.id] = Object.assign(Object.assign(Object.assign({}, buildPage()), page), { 
                                // @ts-ignore it was marked as "special" before
                                type: page.type === "special" ? "system" : page.type, 
                                // remove definitions property
                                properties: {
                                    values: page.properties.values,
                                }, defaultRow: page.defaultRow.type === "row-id"
                                    ? Object.assign(Object.assign({}, page.defaultRow), { rowId: page.defaultRow.rowId ||
                                            (typeof ((_a = page.params) === null || _a === void 0 ? void 0 : _a.rowId) === "string"
                                                ? (_b = page.params) === null || _b === void 0 ? void 0 : _b.rowId
                                                : "") }) : page.defaultRow, params: page.params || {
                                    rowId: ((_c = page.defaultRow) === null || _c === void 0 ? void 0 : _c.type) === "row-id"
                                        ? page.defaultRow.rowId
                                        : "",
                                } });
                        }
                    }
                    const components = Object.fromEntries(Object.entries(app.maps.component).map(([id, component]) => [
                        id,
                        Object.assign(Object.assign({}, component), { showCondition: Array.isArray(component.showCondition)
                                ? showConditionToConditionGroup(component.showCondition)
                                : component.showCondition }),
                    ]));
                    const updatedApp = Object.assign(Object.assign({}, app), { maps: Object.assign(Object.assign({}, app.maps), { page: pages, component: components }), integrations: app.integrations || {}, customSources: app.customSources || {}, theme: Object.assign(Object.assign({}, initialApp.theme), app.theme), schema: Object.assign(Object.assign({}, initialApp.schema), app.schema), components: [...initialApp.components, ...(app.components || [])] });
                    draft.app = updatedApp;
                }, false);
            },
            setIcon: (url) => {
                modify((draft) => {
                    draft.app.icon = url;
                }, false);
            },
            setControl: (control) => {
                modify((draft) => {
                    draft.control = control;
                }, false);
            },
            setInitialPage: (entryPage) => {
                modify((draft) => {
                    draft.app.config.initialPage = Object.assign(Object.assign({}, draft.app.config.initialPage), { entryPage });
                }, false);
            },
            setLoggedInPage: (loggedInPage) => {
                modify((draft) => {
                    draft.app.config.initialPage = Object.assign(Object.assign({}, draft.app.config.initialPage), { loggedInPage });
                }, false);
            },
            addNpmComponent: (name, version) => {
                modify((draft) => {
                    if (draft.app.components.find((c) => c.name === name && c.version === version)) {
                        return;
                    }
                    draft.app.components.push({
                        name,
                        version,
                        enabled: true,
                    });
                });
            },
            removeNpmComponent: (name) => {
                modify((draft) => {
                    const index = draft.app.components.findIndex((c) => c.name === name);
                    if (index !== -1) {
                        draft.app.components.splice(index, 1);
                    }
                });
            },
            setVersionNpmComponent: (name, version) => {
                modify((draft) => {
                    const index = draft.app.components.findIndex((c) => c.name === name);
                    if (index !== -1) {
                        draft.app.components[index].version = version;
                    }
                });
            },
            setEnableNpmComponent: (name, enabled) => {
                modify((draft) => {
                    const index = draft.app.components.findIndex((c) => c.name === name);
                    if (index !== -1) {
                        draft.app.components[index].enabled = enabled;
                    }
                });
            },
            reload: () => {
                modify((draft) => {
                    draft.control = { page: draft.control.page };
                });
            },
            addPage(page) {
                modify((draft) => {
                    draft.app.maps.page[page.id] = page;
                });
            },
            addIntegration(integration) {
                modify((draft) => {
                    draft.app.integrations[integration.name] = integration;
                });
            },
            setIntegrationProperty: function (namespace, field, value) {
                modify((draft) => {
                    draft.app.integrations[namespace].properties.values[field] = value;
                });
            },
            removeIntegration(namespace) {
                modify((draft) => {
                    delete draft.app.integrations[namespace];
                });
            },
            addCustomSources(externalData) {
                modify((draft) => {
                    draft.app.customSources[externalData.name] = externalData;
                });
            },
            removeCustomSources(namespace) {
                modify((draft) => {
                    delete draft.app.customSources[namespace];
                });
            },
            addComponent(component, pageRef, pageOpt) {
                modify((draft) => {
                    draft.app.maps.component[component.id] = component;
                    if (pageRef && pageOpt && typeof pageOpt.idx === "number") {
                        draft.app.maps.page[pageRef.page].components.splice(pageOpt.idx, 0, {
                            component: component.id,
                        });
                    }
                    else if (pageRef) {
                        draft.app.maps.page[pageRef.page].components.push({
                            component: component.id,
                        });
                    }
                });
            },
            addPageAndComponents(page, components) {
                modify((draft) => {
                    draft.app.maps.page[page.id] = page;
                    components.forEach((component) => {
                        draft.app.maps.component[component.id] = component;
                    });
                });
            },
            swapComponent(target, sourceIndex, destinationIndex, deletionLength = 1) {
                modify((draft) => {
                    const components = draft.app.maps.page[target.page].components;
                    const removed = components.splice(sourceIndex, deletionLength);
                    removed && components.splice(destinationIndex, 0, ...removed);
                    draft.app.maps.page[target.page].components = components;
                });
            },
            moveComponents(listUpdateComponents) {
                modify((draft) => {
                    listUpdateComponents.forEach((comp) => {
                        draft.app.maps.component[comp.id].metadata = comp.metadata;
                    });
                });
            },
            swapPage(targetPages, sourceIndex, destinationIndex) {
                modify((draft) => {
                    const pages = Object.values(draft.app.maps.page).filter((page) => page.type !== targetPages[0].type);
                    const [removed] = targetPages.splice(sourceIndex, 1);
                    removed && targetPages.splice(destinationIndex, 0, removed);
                    const newPages = pages.concat(targetPages);
                    draft.app.maps.page = newPages.reduce((reduced, page) => (Object.assign(Object.assign({}, reduced), { [page.id]: page })), {});
                });
            },
            setComponentProperty: function (target, field, value) {
                modify((draft) => {
                    draft.app.maps.component[target.component].properties[field] =
                        value;
                });
            },
            setComponentMetadata: function (target, field, value) {
                modify((draft) => {
                    if (draft.app.maps.component[target.component].metadata) {
                        draft.app.maps.component[target.component].metadata[field] =
                            value;
                    }
                    else {
                        draft.app.maps.component[target.component].metadata = {
                            [field]: value,
                        };
                    }
                });
            },
            setPageProperty: function (target, field, value) {
                modify((draft) => {
                    draft.app.maps.page[target.page].properties.values[field] = value;
                });
            },
            removeComponent(target, from, option) {
                modify((draft) => {
                    var _a;
                    if ((_a = option === null || option === void 0 ? void 0 : option.layoutDetail) === null || _a === void 0 ? void 0 : _a.componentsID) {
                        option.layoutDetail.componentsID.forEach((compId) => {
                            if (draft.app.maps.component[compId].metadata) {
                                draft.app.maps.component[compId].metadata.layout = undefined;
                            }
                            else {
                                draft.app.maps.component[compId].metadata = {};
                            }
                        });
                    }
                    const children = Object.entries(draft.app.maps.component).filter(([, value]) => value.metadata.container === target.component);
                    const childrenKeys = children.map((item) => item[0]);
                    draft.app.maps.page[from.page].components = draft.app.maps.page[from.page].components.filter((ref) => ![...childrenKeys, target.component].includes(ref.component));
                    childrenKeys.forEach((key) => {
                        delete draft.app.maps.component[key];
                    });
                    delete draft.app.maps.component[target.component];
                });
            },
            removeComponents(target, from) {
                modify((draft) => {
                    draft.app.maps.page[from.page].components = draft.app.maps.page[from.page].components.filter((ref) => !target.find((comp) => comp.component === ref.component));
                    target.forEach(({ component }) => {
                        delete draft.app.maps.component[component];
                    });
                });
            },
            removePage(target) {
                modify((draft) => {
                    const page = draft.app.maps.page[target.page];
                    for (const component of page.components) {
                        delete draft.app.maps.component[component.component];
                    }
                    delete draft.app.maps.page[target.page];
                });
            },
            setComponentSource(target, source) {
                modify((draft) => {
                    draft.app.maps.component[target.component].source = source;
                });
            },
            setComponentLayoutMetadata(target, column) {
                modify((draft) => {
                    if (!draft.app.maps.component[target.component].metadata ||
                        !draft.app.maps.component[target.component].metadata.layout)
                        return;
                    draft.app.maps.component[target.component].metadata.layout.column =
                        column;
                });
            },
            removeComponentLayoutMetadata(target) {
                modify((draft) => {
                    if (!draft.app.maps.component[target.component].metadata ||
                        !draft.app.maps.component[target.component].metadata.layout)
                        return;
                    draft.app.maps.component[target.component].metadata.layout =
                        undefined;
                });
            },
            addComponentShowCondition(target) {
                modify((draft) => {
                    if (draft.app.maps.component[target.component]) {
                        draft.app.maps.component[target.component].showCondition.conditions.push({
                            id: nanoid(),
                            formula: "true == true",
                        });
                    }
                });
            },
            updatePage(target, update) {
                modify((draft) => {
                    update(draft.app.maps.page[target.page]);
                });
            },
            updateComponentShowCondition(target, update) {
                modify((draft) => {
                    if (draft.app.maps.component[target.component]) {
                        update(draft.app.maps.component[target.component].showCondition);
                    }
                });
            },
            removeComponentShowCondition(target, index) {
                modify((draft) => {
                    if (draft.app.maps.component[target.component]) {
                        draft.app.maps.component[target.component].showCondition.conditions = draft.app.maps.component[target.component].showCondition.conditions.filter((cond, i) => i !== index);
                    }
                });
            },
            setPageTitle(target, title) {
                modify((draft) => {
                    draft.app.maps.page[target.page].title = title;
                });
            },
        };
    }
    catch (error) { }
}, { enableHistory: true, trackChanges: true, trackPaths: ["app"] });

const defineComponent = (component) => component;
// @ts-ignore
if (typeof window !== "undefined" && !window.qoreApps) {
    // @ts-ignore
    window.qoreApps = Object.assign(Object.assign({}, window.qoreApps), { registerComponent: (id, component) => ({ id, component: Object.assign({}, component) }), registerNpmComponent: (name, component) => ({ name, component }) });
}
const pageFactory = (properties) => {
    return () => ({
        components: [],
        id: nanoid(3),
        properties,
        title: "Untitled page",
        defaultRow: {
            type: "row-id",
            rowId: "",
        },
    });
};
({
    title: { type: "string", options: { format: "text" } },
    onOpen: { type: "action", options: [{ type: "none" }] },
    onScrollToBottom: { type: "action", options: [{ type: "none" }] },
    headerStyle: {
        type: "string",
        options: {
            format: "select",
            options: [
                { label: "None", value: "none" },
                { label: "General", value: "default" },
            ],
        },
    },
    action: { type: "action", options: [{ type: "none" }] },
    icon: {
        type: "string",
        options: {
            format: "icon",
            options: [
                ...Object.keys(iconsList).map((item) => ({
                    label: item,
                    value: item,
                })),
                {
                    label: "None",
                    value: "none",
                },
            ],
        },
    },
    refreshInterval: { type: "number", options: { unit: "seconds" } },
    onSuccess: { type: "action", options: [{ type: "none" }] },
});
const buildPage = pageFactory({
    values: {
        title: "New page",
        onOpen: [{ type: "none" }],
        onScrollToBottom: [{ type: "none" }],
        headerStyle: "default",
        action: [{ type: "none" }],
        icon: "none",
        refreshInterval: 0,
        onSuccess: [{ type: "none" }],
    },
});

const regex = /@\[([^*]+?)]\(([^*]+?)\)/gim;
const format = (text, data, withDisplay = false) => {
    const replaced = (text !== null && text !== void 0 ? text : "")
        .toString()
        .replace(regex, function (substring, display, id) {
        const result = data[id];
        return typeof result !== "undefined" && result !== null
            ? result
            : withDisplay
                ? display
                : "";
    });
    return replaced;
};
memoize(format);

React__default.createContext({
    variables: [],
    values: {},
    componentRef: undefined,
});
memoize(function getVariableValue(variable, values) {
    const variableId = variable.replace(regex, function (_, display, id) {
        return id;
    });
    const result = values[variableId];
    return typeof result !== "undefined" && result !== null ? result : "";
});

const functions = [
    {
        name: "FormatDate",
        description: "format date with given formatting tokens",
        parameters: [
            {
                name: "format",
                type: "string",
                description: "string of formatting tokens",
            },
            { name: "date", type: "date", description: "date to format" },
        ],
        return: {
            type: "string",
            description: "string formatted date",
        },
        evaluate: (format, date) => dayjs(new Date(date)).format(format),
    },
    {
        name: "Filtrex",
        parameters: [
            { name: "expression", type: "string", description: "filtrex expression" },
        ],
        return: {
            type: "any",
            description: "evaluated expression",
        },
        evaluate: (expression) => evaluateExpression(expression),
    },
    {
        name: "FormatCurrency",
        description: "format number into currency with given locale and currency code",
        parameters: [
            { name: "number", type: "number" },
            {
                name: "locale",
                type: "string",
                description: "ISO 639-1 code with optional ISO 3166 code",
            },
            {
                name: "currency",
                type: "string",
                description: "ISO 4217 code",
            },
        ],
        examples: [
            'FormatCurrency(100000, "id", "IDR")',
            'FormatCurrency(100000, "en-US", "USD")',
        ],
        return: {
            type: "string",
            description: "string formatted currency",
        },
        evaluate: (number, locale, currency) => new Intl.NumberFormat(locale, {
            style: "currency",
            currency,
            currencyDisplay: "narrowSymbol",
        }).format(number),
    },
];
const operators = [];
const variables = [];
buildExpressionParser(mergeConfig(javascriptConfig, {
    functions,
    operators,
    variables,
}));
const evaluateExpression = (expression, additionalVariables = []) => {
    const parseExpression = buildExpressionParser(mergeConfig(javascriptConfig, {
        functions,
        operators,
        variables: variables.concat(additionalVariables),
    }));
    const parseResult = parseExpression(expression !== null && expression !== void 0 ? expression : "");
    if (parseResult.errors.length > 0)
        return {
            type: "error",
            errors: parseResult.errors,
        };
    try {
        return {
            type: "ok",
            result: evaluate(parseResult.ast, "javascript", {
                functions,
                operators,
                variables: variables.concat(additionalVariables),
            }),
        };
    }
    catch (e) {
        return { type: "ok", result: "#Error!" };
    }
};

const qoreHooks = createQoreHooks(new QoreClient({
    endpoint: "",
}));
const defaultUser = { status: "idle", error: null, user: null };
const UserContext = React__default.createContext([defaultUser, () => { }]);
const UserFetcher = () => {
    const client = qore.useClient();
    const user = useCurrentUser();
    React__default.useEffect(() => {
        if (user.status === "idle") {
            user.refetch();
        }
    }, [client, user.status, user.refetch]);
    return null;
};
const QoreProvider = (props) => {
    const userContextValue = React__default.useState(defaultUser);
    const contextValue = React__default.useMemo(() => ({ client: props.value }), [props.value]);
    return (jsx(qoreHooks.context.Provider, Object.assign({ value: contextValue }, { children: jsxs(UserContext.Provider, Object.assign({ value: userContextValue }, { children: [jsx(UserFetcher, {}), props.children] })) })));
};
const useCurrentUser = () => {
    const [user, setUser] = React__default.useContext(UserContext);
    const client = qoreHooks.useClient();
    const refetch = React__default.useCallback(() => __awaiter(void 0, void 0, void 0, function* () {
        setUser((user) => (Object.assign(Object.assign({}, user), { status: "loading" })));
        const nextState = yield client
            .currentUser()
            .then((user) => {
            if (typeof user !== "object") {
                return { status: "idle", error: null, user: null };
            }
            return {
                status: "success",
                user,
                error: null,
            };
        })
            .catch((error) => ({
            status: "error",
            user: null,
            error,
        }));
        setUser(nextState);
        return nextState;
    }), [client]);
    const nextUser = React__default.useMemo(() => (Object.assign(Object.assign({}, user), { refetch })), [user, refetch]);
    return nextUser;
};
const qore = Object.assign(Object.assign({}, qoreHooks), { QoreProvider,
    useCurrentUser });
qore.useClient;

/* eslint-disable fp/no-delete -- immer */
var Panel;
(function (Panel) {
    Panel[Panel["Layout"] = 0] = "Layout";
    Panel[Panel["Navigation"] = 1] = "Navigation";
    Panel[Panel["Setting"] = 2] = "Setting";
})(Panel || (Panel = {}));
const initialEditorState = {
    activePanel: Panel.Navigation,
    schema: initialSchema,
    rightPanel: { type: "page" },
    activeWindow: "preview",
    qoreDataConfig: {
        endpoint: "",
        adminSecret: "admin-secret",
    },
    screen: "large",
    editMode: null,
    showEmptyState: {},
    hasProduction: false,
};
createContext(initialEditorState, (modify) => ({
    setActivePanel(panel) {
        modify((draft) => {
            draft.activePanel = panel;
        });
    },
    setRevisionID(revisionID) {
        modify((draft) => {
            draft.revisionID = revisionID;
            draft.loaded = false;
        });
    },
    setActiveComponent(id) {
        modify((draft) => {
            draft.activeComponent = id;
        });
    },
    setActiveCategory(id) {
        modify((draft) => {
            draft.activeCategory = id;
        });
    },
    setRightPanel(name) {
        modify((draft) => {
            draft.rightPanel = name;
        });
    },
    setWindow(name) {
        modify((draft) => {
            draft.activeWindow = name;
        });
    },
    setSchema(schema) {
        modify((draft) => {
            draft.schema = schema;
            draft.loaded = true;
        });
    },
    setEditMode(mode) {
        modify((draft) => {
            draft.editMode = mode;
        });
    },
    setQoreDataConfig(config) {
        modify((draft) => {
            draft.qoreDataConfig = config;
        });
    },
    setScreen(screen) {
        modify((draft) => {
            draft.screen = screen;
        });
    },
    setShowEmptyState(id, show) {
        modify((draft) => {
            draft.showEmptyState[id] = show;
        });
    },
    setHasProduction(toggle) {
        modify((draft) => {
            draft.hasProduction = toggle;
        });
    },
}));

motion(Box);

motion(Box);
React__default.createContext({
    modalPageHistoryLength: 0,
    direction: "none",
    exitDirection: "none",
    setDirection: () => null,
    setExitDirection: () => null,
    forward: () => null,
    back: () => null,
    setModalPageHistoryLength: () => null,
});

const ToastNotificationContext = React__default.createContext({
    notification: null,
    closeNotification: () => { },
    // eslint-disable-next-line @typescript-eslint/no-unused-vars -- for createContext type inference
    triggerNotification: (messages, timeout) => { },
});
React__default.memo(function ToastNotificationContextProvider(props) {
    const [notification, setNotification] = React__default.useState(null);
    const timeoutRef = React__default.useRef(null);
    const closeNotification = React__default.useCallback(() => setNotification(null), []);
    const triggerNotification = React__default.useCallback((message, timeout = 5000) => {
        clearTimeout(timeoutRef.current);
        setNotification(message);
        timeoutRef.current = setTimeout(() => setNotification(null), timeout);
    }, []);
    const contextValue = React__default.useMemo(() => ({
        notification,
        closeNotification,
        triggerNotification,
    }), [notification, closeNotification, triggerNotification]);
    return (jsx(ToastNotificationContext.Provider, Object.assign({ value: contextValue }, { children: props.children })));
});

const EditorToastNotificationContext = React__default.createContext({
    notification: null,
    closeNotification: () => { },
    /* eslint-disable @typescript-eslint/no-unused-vars -- for createContext type inference */
    triggerNotification: (messages, timeout) => { },
    /* eslint-enable */
});
React__default.memo(function EditorToastNotificationContextProvider(props) {
    const [notification, setNotification] = React__default.useState(null);
    const timeoutRef = React__default.useRef(null);
    const closeNotification = React__default.useCallback(() => setNotification(null), []);
    const triggerNotification = React__default.useCallback((message, timeout = 5000) => {
        clearTimeout(timeoutRef.current);
        setNotification(message);
        timeoutRef.current = setTimeout(() => setNotification(null), timeout);
    }, []);
    const contextValue = React__default.useMemo(() => ({
        notification,
        closeNotification,
        triggerNotification,
    }), [notification, closeNotification, triggerNotification]);
    return (jsx(EditorToastNotificationContext.Provider, Object.assign({ value: contextValue }, { children: props.children })));
});

// This component breaks SSR
React__default.lazy(() => import('react-qr-reader'));
const QRScannerActionContext = React__default.createContext({
    showQRScanner: false,
    code: {},
    activeQRScannerId: null,
    closeQRScanner: () => { },
    triggerQRScanner: () => { },
    setQRCode: () => { },
    removeQRCode: () => { },
});
React__default.memo(function QRScannerActionContextProvider({ children, }) {
    const [state, setState] = React__default.useState({
        code: {},
        showQRScanner: false,
        activeQRScannerId: null,
    });
    const triggerQRScanner = React__default.useCallback((activeQRScannerId, metadata, onSuccessAction) => {
        setState((state) => (Object.assign(Object.assign({}, state), { showQRScanner: true, activeQRScannerId, message: (metadata === null || metadata === void 0 ? void 0 : metadata.message) || "", onSuccessAction })));
    }, []);
    const closeQRScanner = React__default.useCallback(() => {
        setState((state) => (Object.assign(Object.assign({}, state), { showQRScanner: false, activeQRScannerId: null, message: undefined, onSuccessAction: undefined })));
    }, []);
    const setQRCode = React__default.useCallback((key, code) => {
        setState((state) => (Object.assign(Object.assign({}, state), { code: Object.assign(Object.assign({}, state.code), { [key]: code }) })));
    }, []);
    const removeQRCode = React__default.useCallback((key) => {
        setState((state) => (Object.assign(Object.assign({}, state), { code: Object.assign(Object.assign({}, state.code), { [key]: null }) })));
    }, []);
    const contextValue = React__default.useMemo(() => ({
        showQRScanner: state.showQRScanner,
        code: state.code,
        activeQRScannerId: state.activeQRScannerId,
        message: state.message,
        onSuccessAction: state.onSuccessAction,
        setQRCode,
        removeQRCode,
        closeQRScanner,
        triggerQRScanner,
    }), [
        state.code,
        state.showQRScanner,
        state.activeQRScannerId,
        state.message,
        state.onSuccessAction,
        setQRCode,
        removeQRCode,
        closeQRScanner,
        triggerQRScanner,
    ]);
    return (jsx(QRScannerActionContext.Provider, Object.assign({ value: contextValue }, { children: children })));
});
function generateScanKeyframes() {
    const key = Array.from(Array(100)).reduce((key, _, i) => key.concat(`
    ${i === 0 ? "from" : i === 100 ? "to" : `${i}%`} {
      transform: translate3d(0,${i}%,0);
    }
    `), "");
    return keyframes `${key}`;
}
generateScanKeyframes();
keyframes `
  from, 20%, 53%, 80%, to {
    transform: translate3d(0,0,0);
  }

  40%, 43% {
    transform: translate3d(0, -20px, 0);
  }

  70% {
    transform: translate3d(0, -10px, 0);
  }

  90% {
    transform: translate3d(0,-4px,0);
  }
`;
keyframes `
  from {
    opacity: 0;
  }

  to {
    opacity: 1;
  }
`;

const globalListeners = new Set();
React.createContext({
    listeners: globalListeners,
    addListener: (listener) => globalListeners.add(listener),
    removeListener: (listener) => globalListeners.delete(listener),
    triggerSubmitHooks: (row) => __awaiter(void 0, void 0, void 0, function* () { return Promise.all([...globalListeners.values()].map((listener) => listener(row))); }),
});

React__default.createContext({
    type: "list",
    params: {},
    sorts: { field: "", type: "asc" },
});
React__default.createContext({
    type: "list",
    params: {},
    sorts: { field: "", type: "asc" },
});

const initialValues = {
    values: {},
    suggestions: [],
    invalidValues: {},
};
createContext(initialValues, (modify) => ({
    handleChange: (key, value) => {
        modify((draft) => {
            draft.values[key] = value;
        });
    },
    setVariables(suggestions) {
        modify((draft) => {
            draft.suggestions = suggestions;
        });
    },
    setValidation: (validation) => {
        modify((draft) => {
            Object.keys(validation).forEach((key) => {
                if (typeof validation[key] === "undefined") {
                    delete draft.invalidValues[key];
                }
                else {
                    draft.invalidValues[key] = validation[key];
                }
            });
        });
    },
    resetValidation: () => {
        modify((draft) => {
            draft.invalidValues = {};
        });
    },
    resetValues: () => {
        modify((draft) => {
            draft.values = {};
        });
    },
}));

React__default.createContext(() => { });

const TextInput = defineComponent({
    name: "Location Input",
    type: "none",
    icon: "LocationInput",
    group: "input",
    display: "inline",
    defaultProps: {
        label: "Location Input",
        hidden: false,
        placeholder: "Placeholder here",
        data: {
            inputID: "location",
            field: { type: "string", options: { format: "text" } },
            validation: {
                required: false,
            },
        },
        defaultValue: "",
    },
    propDefinition: {
        label: { group: "Design", type: "expression", options: {} },
        placeholder: {
            group: "Design",
            type: "expression",
            options: {},
        },
        hidden: { group: "Design", type: "boolean", options: {} },
        data: {
            group: "Design",
            type: "state",
            options: {
                inputID: "location",
                field: { type: "string", options: { format: "text" } },
            },
        },
        defaultValue: {
            group: "Design",
            type: "expression",
            options: {},
        },
    },
    Component: (props) => {
        const title = props.hooks.useEvaluate(props.properties.label);
        const placeholder = props.hooks.useEvaluate(props.properties.placeholder);
        const defaultValue = props.hooks.useEvaluate(props.properties.defaultValue);
        const data = props.hooks.useRuntimeInput(props.properties.data, props.data.page.row, {
            defaultValue,
            validate: (value) => {
                if (!value) {
                    return { error: "Required field, Please enter the data" };
                }
            },
        });
        const handleChange = React__default.useCallback((e) => {
            data.onChange(e.currentTarget.value);
        }, [data.onChange]);
        const [supported, setSupported] = React__default.useState(true);
        const [locating, setLocating] = React__default.useState(true);
        const getLocation = () => {
            if (navigator.geolocation) {
                setLocating(true);
                navigator.geolocation.getCurrentPosition((position) => {
                    data.onChange(`${position.coords.latitude},${position.coords.longitude}`);
                    setLocating(false);
                });
            }
            else {
                setLocating(false);
                setSupported(false);
            }
        };
        React__default.useEffect(() => {
            getLocation();
        }, []);
        return (jsxs(Box, Object.assign({ display: props.properties.hidden ? "none" : undefined }, { children: [jsx(Text, Object.assign({ fontSize: "lg", pb: 2, "data-test": "location-input-title" }, { children: title })), jsxs(InputGroup, Object.assign({ size: "md" }, { children: [jsx(Input, { pr: "10", style: { border: "1px solid rgb(226, 232, 240)" }, bgColor: "surface.900", placeholder: placeholder, name: props.properties.data.inputID, defaultValue: data.value, onChange: handleChange, disabled: supported, _disabled: { border: "1px solid rgb(226, 232, 240)" }, "data-test": "location-input" }, props.properties.data.inputID), jsx(InputRightElement, Object.assign({ width: "10" }, { children: locating ? (jsx(Spinner, {})) : (jsx(IconButton, { "aria-label": "locate", variant: "link", icon: jsx(Icon, { as: TargetLock }), onClick: getLocation, "data-test": "location-input-toggle" })) }))] })), !!data.validation.error && (jsx(Text, Object.assign({ fontSize: "sm", color: "red", "data-test": "location-input-error" }, { children: data.validation.error })))] })));
    },
});

export { TextInput as default };
